------------------------------------------------------------------------------------------
--Power Bar Setup-------------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")

local UnitPowerType = UnitPowerType
local UnitPower = UnitPower
local UnitPowerMax = UnitPowerMax
local BEAR_FORM = BEAR_FORM
local CAT_FORM = CAT_FORM
local SPELL_POWER_MANA = SPELL_POWER_MANA
local SPELL_POWER_RAGE = SPELL_POWER_RAGE
local SPELL_POWER_FOCUS = SPELL_POWER_FOCUS
local SPELL_POWER_ENERGY = SPELL_POWER_ENERGY
local SPELL_POWER_RUNIC_POWER = SPELL_POWER_RUNIC_POWER
local GetSpecialization = GetSpecialization
local GetShapeshiftFormID = GetShapeshiftFormID
local UnitHasVehicleUI = UnitHasVehicleUI
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local GetSpellInfo = GetSpellInfo
local UnitClass = UnitClass
local UnitAura = UnitAura
local unpack = unpack

function SN:SetupPower()
	if F.powermover ~= nil then
		F.powermover = nil
		F.powermover.value = nil
		F.container = nil
		F.power = nil
		F.power.value = nil
	end
	--The powerbar mover frame
	F.powermover = CreateFrame("Frame", nil, UIParent)
	F.powermover:SetPoint(unpack(SN.data.profile.frames.powerbar))
	F.powermover:SetSize(SN.data.profile.powerbar.width, SN.data.profile.powerbar.height)

	--The powerbar mover frame text
	F.powermover.value = F.powermover:CreateFontString(nil, "OVERLAY")
	F.powermover.value:SetPoint("CENTER", F.powermover)
	F.powermover.value:SetFontObject(GameFontNormal)
	F.powermover.value:SetJustifyH("CENTER")

	--The name to display when the powerbar is being moved
	F.powermover.name = "Power Bar"

	--The powerbar statusbar holder
	F.container = CreateFrame("Frame", nil, F.powermover)
	F.container:SetPoint("CENTER", F.powermover)
	F.container:SetSize(SN.data.profile.powerbar.width, SN.data.profile.powerbar.height)

	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(F.container, "Frame")
	else
		F.container:SetBackdrop({
				bgFile = M.media.blankTex, 
				edgeFile = M.media.blankTex, 
				tile = false, tileSize = 0, edgeSize = 2, 
				insets = { left = 2, right = 2, top = 2, bottom = 2 }
				})
		F.container:SetBackdropColor(0.1, 0.1, 0.1, 1)
		F.container:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
		
	end

	--The powerbar statusbar
	F.power = CreateFrame("StatusBar", nil, F.container)
	F.power:SetPoint("CENTER", F.container)

	local powerType = UnitPowerType("player")

	if powerType > 5 then
		powerType = 5
	else
		powerType = powerType + 1
	end
	if (IsAddOnLoaded("ElvUI")) then
		F.power:SetSize(SN.data.profile.powerbar.width - 2, SN.data.profile.powerbar.height - 2)
	else
		F.power:SetSize(SN.data.profile.powerbar.width - 4, SN.data.profile.powerbar.height - 4)
	end
	F.power:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.powerbar.texture))
	F.power:SetStatusBarColor(unpack(M.power[powerType]))
	local min = UnitPower("player")
	local max = UnitPowerMax("player")
	F.power:SetMinMaxValues(0, max)	
	F.power:SetValue(min)

	F.power:SetScript("OnEvent", function(self, event, ...)
		SN:PowerbarEvents(self, event, ...)
	end)
	
	local class, classFileName, classID = UnitClass("player")
	local colortmp = RAID_CLASS_COLORS[classFileName]
	local color = { colortmp.r, colortmp.g, colortmp.b }
	
	--The powerbar statusbar text
	F.power.value = F.power:CreateFontString(nil, "OVERLAY")
	F.power.value:SetPoint("CENTER", F.power)
	F.power.value:SetFont(LSM:Fetch("font", SN.data.profile.powerbar.font), SN.data.profile.powerbar.fontsize, "OUTLINE")
	F.power.value:SetJustifyH("CENTER")
	F.power.value:SetShadowOffset(1, -1)
	F.power.value:SetTextColor(unpack(color))
	if max > 150 then
		F.power.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
	else
		F.power.value:SetText(min)
	end

	--The powerbar events
	F.power:RegisterEvent("PLAYER_ENTERING_WORLD")
	F.power:RegisterEvent("UNIT_POWER_FREQUENT")
	F.power:RegisterEvent("UNIT_MAXPOWER")
	F.power:RegisterEvent("UNIT_DISPLAYPOWER")
	F.power:RegisterEvent("PLAYER_LEVEL_UP")
	
	if SN.data.profile.powerbar.enable then
		SN:PowerEnable()
		if (IsAddOnLoaded("ElvUI")) then
			SN:ElvOverride()
		end
	else
		SN:PowerDisable()
	end
end	

function SN:ShortValue(value)
	if value >= 1e6 then
		return ("%.1fm"):format(value / 1e6):gsub("%.?0+([km])$", "%1")
	elseif value >= 1e3 or value <= -1e3 then
		return ("%.1fk"):format(value / 1e3):gsub("%.?0+([km])$", "%1")
	else
		return value
	end
end

function SN:PowerbarEvents(self, event, ...)

	if event == "PLAYER_ENTERING_WORLD" then
		if self.tick == nil then
			self.tick = CreateFrame("Frame", nil, self)
			self.tick:SetSize(10, self:GetHeight() * 1.6)
			self.tick.tex = self.tick:CreateTexture(nil, "OVERLAY")
			self.tick.tex:SetAllPoints(self.tick)
			self.tick.tex:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")	
			self.tick.tex:SetBlendMode("ADD")
			self.tick:SetAlpha(1)
		end
		local name, _, _, cost = GetSpellInfo(SN.spells[SN.playerClass][GetSpecialization()])
		if name and cost and (cost > 0) then
			self.tick.name = name
			self.tick.cost = cost
		
			local powerType = UnitPowerType("player")
			
			if powerType == 0 then
				self.tick:ClearAllPoints()
				local xoffset = (self:GetValue() - self.tick.cost) * (self:GetWidth() / select(2, self:GetMinMaxValues()) - 5)
				self.tick:SetPoint("LEFT", self, "LEFT", xoffset, 0)
			else
				self.tick:ClearAllPoints()
				local xoffset = (self.tick.cost) * (self:GetWidth() / select(2, self:GetMinMaxValues())) - 5
				self.tick:SetPoint("LEFT", self, "LEFT", xoffset, 0)
			end

			if SN.data.profile.powerbar.tickenable and not self.tick:IsShown() then
				self.tick:Show()
			else
				self.tick:Hide()
			end
		else
			self.tick:Hide()
		end
	end
	
	local powerType = UnitPowerType("player")

	if powerType > 5 then
		powerType = 5
	else
		powerType = powerType + 1
	end
	local min = UnitPower("player")
	local max = UnitPowerMax("player")
	self:SetStatusBarColor(unpack(M.power[powerType]))
	self:SetMinMaxValues(0, max)	
	self:SetValue(min)
	if max > 150 then
		self.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
	else
		self.value:SetText(min)
	end

	if SN.data.profile.powerbar.tickenable then
		local name, _, _, cost = GetSpellInfo(SN.spells[SN.playerClass][GetSpecialization()])
		if name and cost and (cost > 0) then
			self.tick.name = name
			self.tick.cost = cost
		
			local powerType = UnitPowerType("player")
			
			if powerType == 0 then
				self.tick:ClearAllPoints()
				local xoffset = (self:GetValue() - self.tick.cost) * (self:GetWidth() / select(2, self:GetMinMaxValues()) - 5)
				self.tick:SetPoint("LEFT", self, "LEFT", xoffset, 0)
			else
				self.tick:ClearAllPoints()
				local xoffset = (self.tick.cost) * (self:GetWidth() / select(2, self:GetMinMaxValues())) - 5
				self.tick:SetPoint("LEFT", self, "LEFT", xoffset, 0)
			end

			if SN.data.profile.powerbar.tickenable and not self.tick:IsShown() then
				self.tick:Show()
			else
				self.tick:Hide()
			end
		end
	end

	if UnitHasVehicleUI("player") then
		F.container:Hide()
	else
		if not F.container:IsShown() then
			F.container:Show()
		end
	end
end

function SN:PowerEnable()
	F.container:Show()
	F.power:RegisterEvent("UNIT_POWER_FREQUENT")
	F.power:RegisterEvent("UNIT_MAXPOWER")
	F.power:RegisterEvent("UNIT_DISPLAYPOWER")
end

function SN:PowerDisable()
	F.power:UnregisterEvent("UNIT_POWER_FREQUENT")
	F.power:UnregisterEvent("UNIT_MAXPOWER")
	F.power:UnregisterEvent("UNIT_DISPLAYPOWER")
	F.container:Hide()
end